@@ -1,5 +1,4 @@
 /*
- * $Xorg: xfd.c,v 1.4 2001/02/09 02:05:42 xorgcvs Exp $
  *
  *
 Copyright 1989, 1998  The Open Group
@@ -26,12 +25,12 @@ in this Software without prior written authorization from The Open Group.
  * *
  * Author:  Jim Fulton, MIT X Consortium
  */
-/* $XFree86: xc/programs/xfd/xfd.c,v 1.8 2003/02/20 02:56:40 dawes Exp $ */
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
 
+#include <X11/Xfuncproto.h>
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/Xos.h>
@@ -71,10 +70,10 @@ static XrmOptionDescRec xfd_options[] = {
 {"-columns",	"*grid.cellColumns", XrmoptionSepArg,	(caddr_t) NULL },
 };
 
-static void usage(void);
+static void usage(int exitval) _X_NORETURN _X_COLD;
 static void SelectChar(Widget w, XtPointer closure, XtPointer data);
 static void do_quit(Widget w, XEvent *event, String *params,
-		    Cardinal *num_params);
+		    Cardinal *num_params) _X_NORETURN;
 static void change_page(int page);
 static void set_button_state(void);
 static void do_prev(Widget w, XEvent *event, String *params,
@@ -135,7 +134,7 @@ static XtResource Resources[] = {
 #undef Offset
 
 static void
-usage(void)
+usage(int exitval)
 {
     fprintf (stderr, gettext("usage:  %s [-options ...] "), ProgramName);
     fprintf (stderr, "-fn ");
@@ -162,7 +161,7 @@ usage(void)
     fprintf (stderr, gettext("number           number of rows in grid\n"));
     fprintf (stderr, "    -columns ");
     fprintf (stderr, gettext("number        number of columns in grid\n"));
-    exit (1);
+    exit (exitval);
 }
 
 
@@ -185,25 +184,53 @@ main(int argc, char *argv[])
     XftFont *xft;
 #endif
     char *fontname;
-    const char *domaindir;
     long minn, maxn;
 
     XtSetLanguageProc(NULL, NULL, NULL);
 
     ProgramName = argv[0];
 
+    /* Handle args that don't require opening a display */
+    for (int n = 1; n < argc; n++) {
+	const char *argn = argv[n];
+	/* accept single or double dash for -help & -version */
+	if (argn[0] == '-' && argn[1] == '-') {
+	    argn++;
+	}
+	if (strcmp (argn, "-help") == 0) {
+	    usage (0);
+	}
+	if (strcmp (argn, "-version") == 0) {
+	    puts (PACKAGE_STRING);
+	    exit (0);
+	}
+    }
+
     toplevel = XtAppInitialize (&xtcontext, "Xfd",
 				xfd_options, XtNumber(xfd_options),
 				&argc, argv, NULL, NULL, 0);
 
+    if (argc != 1) {
+	fputs(gettext("Unknown argument(s):"), stderr);
+	for (int n = 1; n < argc; n++) {
+	    fprintf(stderr, " %s", argv[n]);
+	}
+	fputs("\n\n", stderr);
+	usage (1);
+    }
+    
 #ifdef USE_GETTEXT
-    textdomain("xfd");
+    {
+	const char *domaindir;
 
-    /* mainly for debugging */
-    if ((domaindir = getenv ("TEXTDOMAINDIR")) == NULL) {
-	domaindir = LOCALEDIR;
+	textdomain("xfd");
+
+        /* mainly for debugging */
+	if ((domaindir = getenv ("TEXTDOMAINDIR")) == NULL) {
+	    domaindir = LOCALEDIR;
+	}
+	bindtextdomain ("xfd", domaindir);
     }
-    bindtextdomain ("xfd", domaindir);
 #endif
 
     Resources[0].default_addr = gettext(DEF_SELECT_FORMAT);
@@ -212,7 +239,6 @@ main(int argc, char *argv[])
     Resources[3].default_addr = gettext(DEF_START_FORMAT);
     Resources[4].default_addr = gettext(DEF_NOCHAR_FORMAT);
 
-    if (argc != 1) usage ();
     XtAppAddActions (xtcontext, xfd_actions, XtNumber (xfd_actions));
     XtOverrideTranslations
         (toplevel, XtParseTranslationTable ("<Message>WM_PROTOCOLS: Quit()"));
@@ -318,11 +344,11 @@ main(int argc, char *argv[])
 
     minn = GridFirstChar (fontGrid);
     maxn = GridLastChar (fontGrid);
-    sprintf (buf, xfd_resources.range_format,
-	     minn >> 8, minn & 0xff,
-	     minn >> 8, minn & 0xff,
-	     maxn >> 8, maxn & 0xff,
-	     maxn >> 8, maxn & 0xff);
+    snprintf (buf, sizeof(buf), xfd_resources.range_format,
+	      minn >> 8, minn & 0xff,
+	      minn >> 8, minn & 0xff,
+	      maxn >> 8, maxn & 0xff,
+	      maxn >> 8, maxn & 0xff);
 
     i = 0;
     XtSetArg (av[i], XtNlabel, buf); i++;
@@ -342,7 +368,7 @@ main(int argc, char *argv[])
 
 /*ARGSUSED*/
 static void
-SelectChar(Widget w, XtPointer closure, XtPointer data)
+SelectChar(Widget w, XtPointer closure _X_UNUSED, XtPointer data)
 {
     FontGridCharRec *p = (FontGridCharRec *) data;
     XFontStruct *fs = p->thefont;
@@ -366,11 +392,11 @@ SelectChar(Widget w, XtPointer closure, XtPointer data)
 	{
 	    XGlyphInfo	extents;
 	    XftTextExtents32 (XtDisplay (w), xft, &c, 1, &extents);
-	    sprintf (buf, xfd_resources.metrics_format,
-		     extents.xOff, - extents.x,
-		     extents.xOff - extents.width + extents.x,
-		     extents.y, extents.height - extents.y,
-		     xft->ascent, xft->descent);
+	    snprintf (buf, sizeof(buf), xfd_resources.metrics_format,
+		      extents.xOff, - extents.x,
+		      extents.xOff - extents.width + extents.x,
+		      extents.y, extents.height - extents.y,
+		      xft->ascent, xft->descent);
 	}
     }
     else
@@ -391,26 +417,26 @@ SelectChar(Widget w, XtPointer closure, XtPointer data)
 	    char2b.byte2 = p->thechar & 0xff;
 	    XTextExtents16 (fs, &char2b, 1, &direction, &fontascent, &fontdescent,
 			    &metrics);
-	    sprintf (buf, xfd_resources.metrics_format,
-		     metrics.width, metrics.lbearing, metrics.rbearing,
-		     metrics.ascent, metrics.descent, fontascent, fontdescent);
+	    snprintf (buf, sizeof(buf), xfd_resources.metrics_format,
+		      metrics.width, metrics.lbearing, metrics.rbearing,
+		      metrics.ascent, metrics.descent, fontascent, fontdescent);
 	}
     }
     XtSetValues (metricsLabel, &arg, ONE);
 
     if (has_char)
     {
-	sprintf (buf, xfd_resources.select_format,
-		 n >> 8, n & 0xff,
-		 n >> 8, n & 0xff,
-		 n >> 8, n & 0xff);
+	snprintf (buf, sizeof(buf), xfd_resources.select_format,
+		  n >> 8, n & 0xff,
+		  n >> 8, n & 0xff,
+		  n >> 8, n & 0xff);
     }
     else
     {
-	    sprintf (buf, xfd_resources.nochar_format,
-		     n >> 8, n & 0xff,
-		     n >> 8, n & 0xff,
-		     n >> 8, n & 0xff);
+	snprintf (buf, sizeof(buf), xfd_resources.nochar_format,
+		  n >> 8, n & 0xff,
+		  n >> 8, n & 0xff,
+		  n >> 8, n & 0xff);
     }
     XtSetValues (selectLabel, &arg, ONE);
 
@@ -420,7 +446,8 @@ SelectChar(Widget w, XtPointer closure, XtPointer data)
 
 /*ARGSUSED*/
 static void
-do_quit (Widget w, XEvent *event, String *params, Cardinal *num_params)
+do_quit (Widget w _X_UNUSED, XEvent *event _X_UNUSED,
+	 String *params _X_UNUSED, Cardinal *num_params _X_UNUSED)
 {
     exit (0);
 }
@@ -430,7 +457,6 @@ change_page(int page)
 {
     long oldstart, newstart;
     int ncols, nrows;
-    char buf[256];
     Arg arg;
 
     arg.name = XtNstartChar;
@@ -452,9 +478,11 @@ change_page(int page)
     if (!page || newstart != oldstart) {
 	unsigned int row = (unsigned int) ((newstart >> 8));
 	unsigned int col = (unsigned int) (newstart & 0xff);
+	char buf[256];
 
 	XtSetArg (arg, XtNlabel, buf);
-	sprintf (buf, xfd_resources.start_format, newstart, row, col);
+	snprintf (buf, sizeof(buf), xfd_resources.start_format,
+		  newstart, row, col);
 	XtSetValues (startLabel, &arg, ONE);
     }
 
@@ -486,14 +514,16 @@ set_button_state(void)
 
 /* ARGSUSED */
 static void
-do_prev16(Widget w, XEvent *event, String *params, Cardinal *num_params)
+do_prev16(Widget w _X_UNUSED, XEvent *event _X_UNUSED,
+	  String *params _X_UNUSED, Cardinal *num_params _X_UNUSED)
 {
     change_page (-16);
 }
 
 
 static void
-do_prev(Widget w, XEvent *event, String *params, Cardinal *num_params)
+do_prev(Widget w _X_UNUSED, XEvent *event _X_UNUSED,
+	String *params _X_UNUSED, Cardinal *num_params _X_UNUSED)
 {
     change_page (-1);
 }
@@ -501,14 +531,16 @@ do_prev(Widget w, XEvent *event, String *params, Cardinal *num_params)
 
 /* ARGSUSED */
 static void
-do_next(Widget w, XEvent *event, String *params, Cardinal *num_params)
+do_next(Widget w _X_UNUSED, XEvent *event _X_UNUSED,
+	String *params _X_UNUSED, Cardinal *num_params _X_UNUSED)
 {
     change_page (1);
 }
 
 /* ARGSUSED */
 static void
-do_next16(Widget w, XEvent *event, String *params, Cardinal *num_params)
+do_next16(Widget w _X_UNUSED, XEvent *event _X_UNUSED,
+	  String *params _X_UNUSED, Cardinal *num_params _X_UNUSED)
 {
     change_page (16);
 }
